org 0x7c00	 ;CUPE OS BOOTLOADER, RAW READ!  Versin 0.02

setup:
mov ax,0x0003
int 0x10

   cli
   mov ax,0xb800
   mov es,ax
   mov sp,0xffff
   sti


a20normal:
cli
call keywait
mov al,0xdf
out 0x64,al
call keywait
xor al,al
in al,0x60
bt ax,1
jc set

mov di,0xA0
mov si,errormsg
call printstring





set:
   xor dx,dx
   xor ax,ax
   int 0x13
   jc set
   mov ax,0x0000
   mov es,ax
   mov bx,0x1000

read.:
	mov ax,0x0217
	mov ch, 0				    ; we are reading the second sector past us, so its still on track 1
	mov cl, 2				    ; sector to read (The second sector)
	mov dh, 0				    ; head number
	xor dx,dx				   ; drive number. Remember Drive 0 is floppy drive.
	int 0x13				    ; call BIOS - Read the sector
	cmp ah,0x00				    ;status
	jne read.
	jmp con






keywait:
xor ax,ax
in al,0x64
bt ax,1
jc keywait
ret

printstring:
   lodsb
   xor al,00
   je printstringf
   mov ah,0x0f
   stosw
   jmp printstring
printstringf:
   ret

con:
   mov ax,0xb800
   mov es,ax
   xor di,di
   mov si,conmsg
   call printstring

setuppmode:
   cli			   ; make sure to clear interrupts first!
   lgdt    [toc]	   ; load GDT into GDTR
   sti
   cli
   mov	   eax, cr0
   or	   eax, 0x1
   mov	   cr0, eax
   jmp	   0x08:Stage3

Stage3:
	;-------------------------------;
	;   preparation                 ;
	;-------------------------------;
	use32
	mov		ax, 0x10		; set data segments to data selector (0x10)
	mov		ss, ax
	mov		es, ax
	mov		ds,ax
	mov fs, ax
	mov gs, ax
	mov		esp, 0x90000		; stack begins from 90000h

	;--------------------------------;
	;PIC, map to 32-47 int number    ;
	;--------------------------------;

	mov	al,0x11
	out	0x20,al
	out	0xa0,al
       ;where to store irq in IVT, from 0-7
	mov	al,0x21 		       ; original: 20
	out	0x21,al
       ;where to store irq in IVT, from 8-15
	mov	al,0x28 		       ; original: 28
	out	0xa1, al

	mov	al, 0x4 		; 0x04 => 0100, second bit (IR line 2)
	out	0x21, al
	mov	al, 0x2 		; 010=> IR line 2
	out	0xa1, al
	mov	al, 1			; bit 0 enables 80x86 mode
	out	0x21, al
	out	0xa1, al
	xor	ax,ax
	out	0x21, al
	out	0xa1, al

	jmp 0x1000	    ; jump next sector












errormsg:db "Error during startup",0x00
conmsg:db "Starting CUPE-OS",0x00


; Offset 0 in GDT: Descriptor code=0

gdt_data: 
	dd 0				; null descriptor
	dd 0 

; Offset 0x8 bytes from start of GDT: Descriptor code therfore is 8

; gdt code:                             ; code descriptor
	dw 0FFFFh			; limit low
	dw 0				; base low
	db 0				; base middle
	db 10011010b			; access
	db 11001111b			; granularity
	db 0				; base high

; Offset 16 bytes (0x10) from start of GDT. Descriptor code therfore is 0x10.

; gdt data:                             ; data descriptor
	dw 0FFFFh			; limit low (Same as code)
	dw 0				; base low
	db 0				; base middle
	db 10010010b			; access
	db 11001111b			; granularity
	db 0				; base high

;...Other descriptors begin at offset 0x18. Remember that each descriptor is 8 bytes in size?
; Add other descriptors for Ring 3 applications, stack, whatever here...

end_of_gdt:
toc: 
	dw end_of_gdt - gdt_data - 1	; limit (Size of GDT)
	dd gdt_data			; base of GDT



rb 7E00h-2-$
db 0x55,0xaa

